@@ -16,6 +16,7 @@ import glob
 import io
 import json
 import logging
+import logging.config
 import os
 import platform
 import re
@@ -105,8 +106,39 @@ KERNEL_NAME_PATTERN = re.compile(r"^[a-z0-9._\-]+$", re.IGNORECASE)
 # Create a structure to store information about the location of a kernel on the current machine.
 _kernel_info = collections.namedtuple(typename="kernel_info", field_names=("name", "path"))
 
-# Create a new instance of the preferred reporting system.
-_logger = logging.getLogger(__name__)
+LOGGING_CONFIG = {
+    "formatters": {
+        "default": {
+            "format": "[%(levelname)s] %(name)s: %(message)s",
+        },
+    },
+    "handlers": {
+        "logfile": {
+            "class": "logging.FileHandler",
+            "encoding": "utf-8",
+            "filename": "/tmp/notebook-environments.log",
+            "formatter": "default",
+            "mode": "wt",
+        },
+        "stdout": {
+            "class": "logging.StreamHandler",
+            "formatter": "default",
+            "stream": "ext://sys.stdout",
+        },
+    },
+    "loggers": {
+        "notebook-environments": {
+            "handlers": ["logfile", "stdout"],
+            # Don't send it up this namespace for additional handling.
+            "propagate": False,
+        },
+    },
+    # Set the preferred schema version.
+    "version": 1,
+}
+logging.config.dictConfig(config=LOGGING_CONFIG)
+# Create a new instance of the preferred reporting system for this program.
+_logger = logging.getLogger("notebook-environments")
 
 
 __all__ = (
@@ -117,7 +149,7 @@ __all__ = (
     "show_kernels",
 )
 
-__version__ = "0.8.7"
+__version__ = "0.8.9"
 
 
 def _in_virtual_environment():
@@ -198,6 +230,7 @@ def _write_kernel_specification(path):
             stream_out.write(_to_unicode(json.dumps(kernel_spec, ensure_ascii=False, indent=2)))
     except (IOError, OSError) as err:
         _logger.error("It's impossible to create a new specification on the current machine.")
+        _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
         # Stop this program runtime and return the exit status code.
         sys.exit(getattr(err, "errno", errno.EIO))
 
@@ -216,6 +249,7 @@ def _provide_required_packages():
                 "You are to update setup tools and run the installation process another time.\n"
                 "python -m pip install --upgrade pip setuptools wheel"
             ))
+            _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
             # Stop this program runtime and return the exit status code.
             sys.exit(getattr(err, "errno", errno.EPERM))
 
@@ -229,11 +263,13 @@ def _write_python_logos(path):
     for logo_name, logo_image in logos_spec.items():
         try:
             with io.open(os.path.join(path, logo_name), mode="wb") as stream_out:
+                # Skip this step when an unexpected error has occurred.
                 with contextlib.suppress(binascii.Error):
                     # Create a new python logo on the current machine.
                     stream_out.write(base64.b64decode(logo_image))
         except (IOError, OSError) as err:
             _logger.error("It's impossible to create python logos on the current machine.")
+            _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
             # Stop this program runtime and return the exit status code.
             sys.exit(getattr(err, "errno", errno.EIO))
 
@@ -262,6 +298,7 @@ def _create_dir(path):
             pass
         else:
             _logger.error("It's impossible to create a new directory on the current machine.")
+            _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
             # Stop this program runtime and return the exit status code.
             sys.exit(getattr(err, "errno", errno.EPERM))
 
@@ -275,6 +312,7 @@ def _remove_dir(path):
             shutil.rmtree(path)
     except OSError as err:
         _logger.error("It's impossible to remove a directory from the current machine.")
+        _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
         # Stop this program runtime and return the exit status code.
         sys.exit(getattr(err, "errno", errno.EPERM))
 
@@ -316,6 +354,7 @@ def purge_broken_kernels():
             _check_and_remove_broken_kernel(kernel_info.path)
     except OSError as err:
         _logger.error("It's impossible to find and remove broken python kernels.")
+        _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
         # Stop this program runtime and return the exit status code.
         sys.exit(getattr(err, "errno", errno.EPERM))
 
@@ -348,6 +387,7 @@ def show_kernels():
             print("kernel: {0} --> {1}".format(kernel_info.name, kernel_info.path))
     except OSError as err:
         _logger.error("It's impossible to show python kernels on the working notebook server.")
+        _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
         # Stop this program runtime and return the exit status code.
         sys.exit(getattr(err, "errno", errno.EPERM))
 
@@ -359,6 +399,15 @@ def main():  # pragma: no cover
         description="Manage python virtual environments on the working notebook server."
     )
     parser.add_argument("-v", "--version", action="version", version=str(__version__))
+    parser.add_argument(
+        "-d",
+        "--debug",
+        action="store_const",
+        const=logging.DEBUG,  # level must be an int or a str
+        default=logging.WARNING,
+        dest="logging_level",
+        help="print a lot of debugging statements while executing user's commands",
+    )
 
     # Prevent the users from using two or more arguments at this program runtime.
     group = parser.add_mutually_exclusive_group(required=True)
@@ -367,7 +416,7 @@ def main():  # pragma: no cover
         "--add",
         action="store_const",
         const=add_active_environment,
-        dest="command",
+        dest="user_command",
         help="add an active python environment to the working notebook server",
     )
     group.add_argument(
@@ -375,7 +424,7 @@ def main():  # pragma: no cover
         "--remove",
         action="store_const",
         const=remove_active_environment,
-        dest="command",
+        dest="user_command",
         help="find and remove an active python environment from the working notebook server",
     )
     group.add_argument(
@@ -383,7 +432,7 @@ def main():  # pragma: no cover
         "--purge",
         action="store_const",
         const=purge_broken_kernels,
-        dest="command",
+        dest="user_command",
         help="find and remove broken python kernels from the working notebook server",
     )
     group.add_argument(
@@ -391,7 +440,7 @@ def main():  # pragma: no cover
         "--initialize",
         action="store_const",
         const=initialize_new_notebook_environment,
-        dest="command",
+        dest="user_command",
         help="find and remove all python kernels and add the working interpreter",
     )
     group.add_argument(
@@ -399,15 +448,21 @@ def main():  # pragma: no cover
         "--show",
         action="store_const",
         const=show_kernels,
-        dest="command",
+        dest="user_command",
         help="show active python kernels on the working notebook server",
     )
 
     try:
+        arguments = parser.parse_args()
+
+        # Set a new logging level of the preferred reporting system.
+        _logger.setLevel(arguments.logging_level)
+
         # Execute a command on the current machine.
-        parser.parse_args().command()
+        arguments.user_command()
     except KeyboardInterrupt as err:
         _logger.error("Stop this program runtime on the current machine.")
+        _logger.debug("An unexpected error occurred at this program runtime:", exc_info=True)
         # Stop this program runtime and return the exit status code.
         sys.exit(getattr(err, "errno", errno.EINTR))
 
